############################################################
# We fit a regression tree to Y using two inputs
# The function rpart in the library rpart
library(rpart)
###########################################################################
br = read.table("br.csv", sep = ",", header=T) # reads the data from a csv file                               
# We randomly select a training sample of N=600 observations
Ntot = nrow(br);  # total n. of obs.
 
N = 700;          # size of training sample
s = sample(1:Ntot, N);  # index of selected units
train = br[s,];   # training sample
test = br[-s,];   # validation sample
summary(train);
attach(train);
###########################################################################
x1 = sqft; 
x2 = Age;
y  = price;
# Regression tree using rpart
rtree = rpart( price~sqft , method ='anova');
# plot the tree #########################
windows()
par(mfrow=c(1,2))
plot(rtree, uniform=T); 
text(rtree,use.n=T,all=T, cex=0.65, col = 'red')
plotcp(rtree, col='blue')   #plot of GOF vs complexity parameter
x1p = seq(min(x1), max(x1), length = 50)
fit = predict(rtree ) 
plot(x1,y)
plot(x1, fit)
######################################################
# Regression tree using rpart, two variables
rtree = rpart(y~x1+x2, method ='anova', control = rpart.control(cp = 0.01))
# plot the tree #########################
windows()
par(mfrow=c(1,2))
plot(rtree, uniform=T); 
text(rtree,use.n=T,all=T, cex=0.65, col = 'red')
plotcp(rtree, col='blue')   #plot cross-validation results
########################
x1p = seq(min(x1), max(x1), length = 55)
np = length(x1p)
x2p = seq(min(x2), max(x2), length = 55)
pt = expand.grid(x1 = x1p, x2 = x2p)
fit = predict(rtree, pt  ) 
windows()
graph = persp(x=x1p, y=x2p, matrix(fit, np, np),
               xlab="x1", ylab="x2", zlab="y", expand=0.5,
               col="darkgreen", ticktype="detailed", theta= -45)
###############################################################################
# RANDOM FORESTS
#install.packages("randomForest")
library(randomForest)
# ================== random forest
rf = randomForest(y~x1+x2, ntree=90, mtry = 1, # number of variables sampled 
                  importance=TRUE)
print(rf)
importance(rf)
getTree(rf, k= 1, labelVar=TRUE)
getTree(rf,k = 22)
windows()
par(mfrow=c(1,2))
partialPlot(rf, train, x.var =  "sqft")
partialPlot(rf, train, x.var =  "Age")
fit = predict(rf, pt ) 
windows()
graph = persp(x=x1p, y=x2p, matrix(fit, np, np),
              xlab="x1", ylab="x2", zlab="y", expand=0.5,
              col="darkgreen", ticktype="detailed", theta= -45)